import io
import os
from typing import Any

from prefect import task
import pandas as pd

from io_collection.save.save_buffer import save_buffer_to_s3


@task
def save_dataframe(location: str, key: str, dataframe: pd.DataFrame, **kwargs: Any) -> None:
    if location[:5] == "s3://":
        save_dataframe_to_s3(location[5:], key, dataframe, **kwargs)
    else:
        save_dataframe_to_fs(location, key, dataframe, **kwargs)


def save_dataframe_to_fs(path: str, key: str, dataframe: pd.DataFrame, **kwargs: Any) -> None:
    full_path = os.path.join(path, key)
    os.makedirs(os.path.split(full_path)[0], exist_ok=True)
    dataframe.to_csv(full_path, **kwargs)


def save_dataframe_to_s3(bucket: str, key: str, dataframe: pd.DataFrame, **kwargs: Any) -> None:
    """
    Saves dataframe to bucket with given key.
    """
    with io.BytesIO() as buffer:
        dataframe.to_csv(buffer, **kwargs)
        save_buffer_to_s3(bucket, key, buffer)



import os

import boto3
from prefect import task


@task
def check_key_exists(location: str, key: str) -> bool:
    if location[:5] == "s3://":
        return check_key_exists_s3(location[5:], key)
    else:
        return check_key_exists_fs(location, key)


def check_key_exists_fs(path: str, key: str) -> bool:
    full_path = os.path.join(path, key)
    return os.path.isfile(full_path)


def check_key_exists_s3(bucket: str, key: str) -> bool:
    s3_client = boto3.client("s3")
    result = s3_client.list_objects(Bucket=bucket, Prefix=key)
    return "Contents" in result



import io
import os

import boto3
from prefect import task


@task
def save_buffer(location: str, key: str, buffer: io.BytesIO) -> None:
    if location[:5] == "s3://":
        save_buffer_to_s3(location[5:], key, buffer)
    else:
        save_buffer_to_fs(location, key, buffer)


def save_buffer_to_fs(path: str, key: str, buffer: io.BytesIO) -> None:
    full_path = os.path.join(path, key)
    os.makedirs(os.path.split(full_path)[0], exist_ok=True)
    with open(full_path, "wb") as file:
        file.write(buffer.getvalue())


def save_buffer_to_s3(bucket: str, key: str, buffer: io.BytesIO) -> None:
    """
    Saves buffer to bucket with given key.
    """
    s3_client = boto3.client("s3")
    s3_client.put_object(Bucket=bucket, Key=key, Body=buffer.getvalue())



import io
import os

import boto3
from prefect import task

MAX_CONTENT_LENGTH = 2**31 - 1


@task
def load_buffer(location: str, key: str) -> io.BytesIO:
    if location[:5] == "s3://":
        return load_buffer_from_s3(location[5:], key)
    else:
        return load_buffer_from_fs(location, key)


def load_buffer_from_fs(path: str, key: str) -> io.BytesIO:
    full_path = os.path.join(path, key)
    return io.BytesIO(open(full_path, "rb").read())


def load_buffer_from_s3(bucket: str, key: str) -> io.BytesIO:
    """
    Loads body from bucket for given key.
    """
    s3_client = boto3.client("s3")
    obj = s3_client.get_object(Bucket=bucket, Key=key)

    # Check if body needs to be loaded in chunks.
    content_length = obj["ContentLength"]

    if content_length > MAX_CONTENT_LENGTH:
        print("Loading chunks ...")
        body = bytearray()
        for chunk in obj["Body"].iter_chunks(chunk_size=MAX_CONTENT_LENGTH):
            body += chunk
        return io.BytesIO(body)
    else:
        return io.BytesIO(obj["Body"].read())



import os
from datetime import datetime

from prefect import task


@task
def make_key(*subkeys: str) -> str:
    key = os.path.join(*subkeys)

    timestamp = datetime.now().strftime("%Y-%m-%d")
    key = key.replace("{{timestamp}}", timestamp)

    return key



import io
import os

from prefect import task

from io_collection.save.save_buffer import save_buffer_to_s3


@task
def save_text(location: str, key: str, text: str) -> None:
    if location[:5] == "s3://":
        save_text_to_s3(location[5:], key, text)
    else:
        save_text_to_fs(location, key, text)


def save_text_to_fs(path: str, key: str, text: str) -> None:
    full_path = os.path.join(path, key)
    os.makedirs(os.path.split(full_path)[0], exist_ok=True)
    with open(full_path, "w", encoding="utf-8") as file:
        file.write(text)


def save_text_to_s3(bucket: str, key: str, text: str) -> None:
    with io.BytesIO() as buffer:
        buffer.write(text.encode("utf-8"))
        save_buffer_to_s3(bucket, key, buffer)



import os

from prefect import task

from io_collection.load.load_buffer import load_buffer_from_s3


@task
def load_text(location: str, key: str) -> str:
    if location[:5] == "s3://":
        return load_text_from_s3(location[5:], key)
    else:
        return load_text_from_fs(location, key)


def load_text_from_fs(path: str, key: str) -> str:
    full_path = os.path.join(path, key)
    return open(full_path, "r", encoding="utf-8").read()


def load_text_from_s3(bucket: str, key: str) -> str:
    buffer = load_buffer_from_s3(bucket, key)
    return buffer.getvalue().decode("utf-8")



import os
import tarfile

from prefect import task

from io_collection.load.load_buffer import load_buffer_from_s3


@task(persist_result=False)
def load_tar(location: str, key: str) -> tarfile.TarFile:
    if location[:5] == "s3://":
        return load_tar_from_s3(location[5:], key)
    else:
        return load_tar_from_fs(location, key)


def load_tar_from_fs(path: str, key: str) -> tarfile.TarFile:
    full_path = os.path.join(path, key)
    return tarfile.open(full_path, mode="r:xz")


def load_tar_from_s3(bucket: str, key: str) -> tarfile.TarFile:
    buffer = load_buffer_from_s3(bucket, key)
    return tarfile.open(fileobj=buffer, mode="r:xz")



import os
from typing import Any

import pandas as pd
from prefect import task

from io_collection.load.load_buffer import load_buffer_from_s3


@task
def load_dataframe(location: str, key: str, **kwargs: Any) -> pd.DataFrame:
    if location[:5] == "s3://":
        return load_dataframe_from_s3(location[5:], key, **kwargs)
    else:
        return load_dataframe_from_fs(location, key, **kwargs)


def load_dataframe_from_fs(path: str, key: str, **kwargs: Any) -> pd.DataFrame:
    full_path = os.path.join(path, key)
    return pd.read_csv(full_path, **kwargs)


def load_dataframe_from_s3(bucket: str, key: str, **kwargs: Any) -> pd.DataFrame:
    buffer = load_buffer_from_s3(bucket, key)
    return pd.read_csv(buffer, **kwargs)



import os
from glob import glob

import boto3
from prefect import task


@task
def get_keys(location: str, prefix: str) -> list[str]:
    if location[:5] == "s3://":
        return get_keys_s3(location[5:], prefix)
    else:
        return get_keys_fs(location, prefix)


def get_keys_fs(path: str, prefix: str) -> list[str]:
    all_files = glob(f"{path}{prefix}/**/*", recursive=True)
    regular_files = [file for file in all_files if not os.path.isdir(file)]
    keys = [file.replace(path, "") for file in regular_files]
    return keys


def get_keys_s3(bucket: str, prefix: str) -> list[str]:
    s3_client = boto3.client("s3")

    bucket = bucket.replace("s3://", "")
    prefix = f"{prefix}/"
    response = s3_client.list_objects_v2(Bucket=bucket, Prefix=prefix)

    get_response = True
    keys: list[str] = []

    while get_response:
        if "Contents" not in response:
            break

        content_keys = [content["Key"] for content in response["Contents"]]
        keys = keys + content_keys

        if response["IsTruncated"]:
            response = s3_client.list_objects_v2(
                Bucket=bucket,
                Prefix=prefix,
                ContinuationToken=response["NextContinuationToken"],
            )
        else:
            get_response = False

    return keys
